@@ -6,9 +6,9 @@ use warnings;
 use Log::ger;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2025-10-31'; # DATE
+our $DATE = '2025-11-26'; # DATE
 our $DIST = 'App-FindUtils'; # DIST
-our $VERSION = '0.004'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
 our %SPEC;
 
@@ -33,6 +33,36 @@ $SPEC{find_duplicate_filenames} = {
             schema => 'bool*',
             cmdline_aliases => {l=>{}},
         },
+        eval => {
+            #schema => 'code_from_str::local_topic*', # not yet available
+            schema => 'str*',
+            description => <<'MARKDOWN',
+
+Process filename through this code. Code will receive filename in `$_` and is
+expected to change and return a new "name" that will be compared for duplicate
+instead of the original name. You can use this e.g. to find duplicate in some
+part of the filename. As an alternative, see the `--regex` option.
+
+MARKDOWN
+            cmdline_aliases => {e=>{}},
+        },
+        regex => {
+            schema => 're_from_str*',
+            description => <<'MARKDOWN',
+
+Specify a regex with a capture to get part of the filename. The first capture
+`$1` will be used to compare for duplicate instead of the original name. You can
+use this to find duplicate in some part of the filename. As an alternative, see
+the `--eval` option.
+
+MARKDOWN
+            cmdline_aliases => {r=>{}},
+        },
+        exclude_filename_regex => {
+            schema => 're_from_str*',
+            summary => 'Filename regex to exclude',
+            cmdline_aliases => {x=>{}},
+        },
     },
     examples => [
         {
@@ -42,7 +72,17 @@ $SPEC{find_duplicate_filenames} = {
             src => '[[prog]]',
             src_plang => 'bash',
         },
+        {
+            summary => "Find duplicate receipts by order ID (filenames are named receipt-order=12345.pdf), exclude backup files",
+            test => 0,
+            'x.doc.show_result' => 0,
+            src => q{[[prog]] -x '/\\.bak$/' -r '/order=(\\d+)/' --debug},
+            src_plang => 'bash',
+        },
     ],
+    args_rels => {
+        choose_one => ['eval', 'regex'],
+    },
 };
 sub find_duplicate_filenames {
     require Cwd;
@@ -50,30 +90,53 @@ sub find_duplicate_filenames {
 
     my %args = @_;
     $args{dirs} //= ["."];
+    my $eval;
+    if (defined $args{eval}) {
+        my $code = "no strict; no warnings; package main; sub { local \$_=\$_; " . $args{eval} . "; return \$_ }";
+        $eval = eval $code or return [400, "Can't compile code in eval: $@"]; ## no critic: BuiltinFunctions::ProhibitStringyEval
+    } elsif (defined $args{regex}) {
+        $eval = sub { /$args{regex}/; $1 };
+    }
+
     #my $ci = $args{case_insensitive};
 
-    my %files; # filename => {realpath1=>orig_filename, ...}. if hash has >1 keys than it's duplicate
+    my %names; # filename (or name) => {realpath1=>1, ...}. if hash has >1 keys than it's duplicate
     File::Find::find(
         sub {
             no warnings 'once'; # for $File::find::dir
             # XXX inefficient
             my $realpath = Cwd::realpath($_);
             log_debug "Found path $realpath";
-            $files{$_}{$realpath}++;
+
+            if ($args{exclude_filename_regex}) {
+                if ($_ =~ $args{exclude_filename_regex}) {
+                    log_info "$_ excluded (matches --exclude-filename-regex: $args{exclude_filename_regex})";
+                    return;
+                }
+            }
+
+            my $name;
+            if ($eval) {
+                $name = $eval->();
+            } else {
+                $name = $_;
+            }
+
+            $names{$name}{$realpath}++;
         },
         @{ $args{dirs} }
     );
 
     my @res;
-    for my $file (sort keys %files) {
-        next unless keys(%{$files{$file}}) > 1;
-        log_info "%s is a DUPLICATE name (found in %d paths)", $file, scalar(keys %{$files{$file}});
+    for my $name (sort keys %names) {
+        next unless keys(%{$names{$name}}) > 1;
+        log_info "%s is a DUPLICATE name (found in %d paths: %s)", $name, scalar(keys %{$names{$name}}), join(", ", sort(keys %{$names{$name}}));
         if ($args{detail}) {
-            for my $path (sort keys %{$files{$file}}) {
-                push @res, {name=>$file, path=>$path};
+            for my $path (sort keys %{$names{$name}}) {
+                push @res, {name=>$name, path=>$path};
             }
         } else {
-            push @res, $file;
+            push @res, $name;
         }
     }
     [200, "OK", \@res];
@@ -94,7 +157,7 @@ App::FindUtils - Utilities related to finding files
 
 =head1 VERSION
 
-This document describes version 0.004 of App::FindUtils (from Perl distribution App-FindUtils), released on 2025-10-31.
+This document describes version 0.005 of App::FindUtils (from Perl distribution App-FindUtils), released on 2025-11-26.
 
 =head1 DESCRIPTION
 
@@ -131,6 +194,24 @@ Instead of just listing duplicate names, return all the location of duplicates.
 
 (No description)
 
+=item * B<eval> => I<str>
+
+Process filename through this code. Code will receive filename in C<$_> and is
+expected to change and return a new "name" that will be compared for duplicate
+instead of the original name. You can use this e.g. to find duplicate in some
+part of the filename. As an alternative, see the C<--regex> option.
+
+=item * B<exclude_filename_regex> => I<re_from_str>
+
+Filename regex to exclude.
+
+=item * B<regex> => I<re_from_str>
+
+Specify a regex with a capture to get part of the filename. The first capture
+C<$1> will be used to compare for duplicate instead of the original name. You can
+use this to find duplicate in some part of the filename. As an alternative, see
+the C<--eval> option.
+
 
 =back
 
